import {
  Datasource,
  IEventParam,
  Metadata,
  Http,
} from '@zeedhi/core';
import {
  TekGrid,
} from '@zeedhi/teknisa-components-common';
import {
  Button,
  GridEditable,
  LoadingService,
  Text,
} from '@zeedhi/common';
import { Utilities } from '@/util/Utilities';

export class PercentageAndStructureForBudgetController {
  // private departmentCurrentRow: any = {};
  
  private employeeCurrentRow: any = {};
  
  public isNotUpdateRowEditing = true;
  
  public isNotInlineSelectsWithRequestEditing = true;
  
  public previewStructure = 1;
  
  public routeDescription = '';

  public onInlineEditUpdateRow({ component }: IEventParam<GridEditable>) {
    this.isNotUpdateRowEditing = !component.editing;
  }

  public async cancelEditingUpdateRow({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    await grid.cancelEditedRows();
    this.onInlineEditUpdateRow({ component: grid });
    grid.editingNewRows = false;
  }

  public async saveEditingUpdateRow({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    await grid.saveEditedRows();
    this.onInlineEditUpdateRow({ component: grid });
    grid.editingNewRows = false;
  }

  get isInlineSelectsWithRequestEditing() {
    return !this.isNotInlineSelectsWithRequestEditing;
  }

  public onInlineEditSelectsWithRequest({ component }: IEventParam<GridEditable>) {
    this.isNotInlineSelectsWithRequestEditing = !component.editing;
  }

  public async cancelEditingSelectsWithRequest({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    await grid.cancelEditedRows();
    this.onInlineEditSelectsWithRequest({ component: grid });
  }

  public async saveEditingSelectsWithRequest({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    await grid.saveEditedRows();
    this.onInlineEditSelectsWithRequest({ component: grid });
  }
  
  public addNewRow({ component }: any) {
    const gridDetailItens = Metadata.getInstance<TekGrid>('detailsOperationalItems');
    const grid: GridEditable = component.parent;
    const newRow = {
    };
    console.warn(gridDetailItens);
    grid.columns[2].editable = true;
    grid.editingNewRows = true;
    grid.addNewRow(newRow);
  }
  
  public structureItemChange({ component }: IEventParam<Button>) {
    const utilities = new Utilities();
    const grid = component.parent as TekGrid;
    const data = grid.getEditedRows();
    LoadingService.show();
    Http.post('/structureItemChange', {
      requestType: 'DataSet',
      dataset: data,
    }).then(async () => {
      await grid.saveEditedRows();
      this.onInlineEditUpdateRow({ component: grid });
      LoadingService.hide();
      utilities.successMessage();
    }).catch((e) => {
      utilities.errorMessage();
      LoadingService.hide();
      console.warn(e);
    });
  }
  
  public unitRowClick({ component }: any) {
    const selectedUnit = `${component.datasource.currentRow.CDFILIAL} - ${component.datasource.currentRow.NMFILIAL}`;
    const routeName = 'operationalItemStructure';
    const grid = Metadata.getInstance<TekGrid>('gridOperationalItems');
    const unit = Metadata.getInstance<Text>('selectedUnit');
    
    const sendObject = {
      filter: [{
        name: 'CDFILIAL',
        operator: '=',
        value: component.datasource.currentRow.CDFILIAL,
      }],
    };
    let data;
    
    LoadingService.show();
    Http.post(`/${routeName}`, sendObject).then((response) => {
      data = response.data.dataset[routeName];
      
      /*
      if (data.some((obj) => ('IDTIPO' in obj))) {
        data.forEach((obj) => {
          obj.IDTIPO = I18n.translate('gop::T_'.concat(obj.IDTIPO));
        });
      }
      */
      grid.datasource.setFilter([]);
      
      grid.datasource.updateData(data);
      
      LoadingService.hide();
    }).catch((e) => {
      // utilities.errorMessage(I18n.translate('gop::FAILED_DATA'));
      LoadingService.hide();
      console.warn(e);
    });
    
    unit.text = selectedUnit;
  }
  
  public rowOperationalItemClick({ component }: any) {
    const routeName = 'detailsOperationalItemsStructure';
    const grid = Metadata.getInstance<TekGrid>('detailsOperationalItems');
    const { currentRow } = component.datasource;
    if (currentRow.NRSEQ === 11) {
      component.parent.height = '96%';
      grid.parent.isVisible = false;
    } else {
      const sendObject = {
        filter: [{
          name: 'NRSEQITEMOP',
          operator: '=',
          value: currentRow.NRSEQ,
        },
        {
          name: 'CDFILIAL',
          operator: '=',
          value: currentRow.CDFILIAL,
        }],
      };
      // this.defineRouteDescription(currentRow.NRSEQITEMORCA);
      LoadingService.show();
      Http.post(`/${routeName}`, sendObject).then((response) => {
        const data = response.data.dataset[routeName];
        
        /*
        if (data.some((obj) => ('IDTIPO' in obj))) {
          data.forEach((obj) => {
            obj.IDTIPO = I18n.translate('gop::T_'.concat(obj.IDTIPO));
          });
        }
        */
        grid.datasource.setFilter([]);
        grid.datasource.updateData(data);
        LoadingService.hide();
      }).catch((e) => {
        // utilities.errorMessage(I18n.translate('gop::FAILED_DATA'));
        LoadingService.hide();
        console.warn(e);
      });
      
      component.parent.height = '30%';
      grid.parent.isVisible = true;
    }
  }
  
  /* public defineRouteDescription(nrSeqItemOrca) {
    const routeObject = [
      // FATURAMENTO
      {NRSEQITEM:0, route: '/crudN/descriptionBilling', params: []},
      // IMPOSTO
      {NRSEQITEM:1, route: '/crudN/descriptionTax', params: []},
      // CONSUMO
      {NRSEQITEM:2, route: '/crudN/descriptionConsumption',params: []},
      // MAO DE OBRA
      {NRSEQITEM:3, route: '/crudN/descriptionLabor',params: []},
      // DESPESAS DA MATRIZ
      {NRSEQITEM:5, route: '/crudN/descriptionHeadOfficeExpenses',params: []},
      // DESPESAS DIVERSAS
      {NRSEQITEM:6, route: '/crudN/descriptionMiscellaneousExpenses',params: []},
      // DESPESAS LANCAMENTO ENTRADA
      {NRSEQITEM:7, route: '/crudN/descriptionExpensesEntryRelease',params: []},
      // IMPOSTO A RECUPERAR
      {NRSEQITEM:8, route: '/crudN/descriptionTaxesToBeRecovered',params: []},
      // IMPOSTO SOBRE O LUCRO
      {NRSEQITEM:9, route: '/crudN/descriptionTaxesOnProfit',params: []},
      // INVESTIMENTOS
      {NRSEQITEM:10, route: '/crudN/descriptionInvestment',params: []},
      // DEPRECIACAO
      {NRSEQITEM:11, route: '/crudN/descriptionDepreciation',params: []},
      //SERVICO
      {NRSEQITEM:11, route: '/crudN/descriptionService',params: []},
    ];
    this.routeDescription = routeObject[nrSeqItemOrca];
  } */
  
  public onChangeEmployeeCurrentRow({ component }: IEventParam<Datasource>) {
    this.employeeCurrentRow = component.currentRow;
  }
  
  public onChangePreviewStructure({ component }) {
    if (component.datasource.currentRowValue) {
      this.previewStructure = component.datasource.currentRowValue.value;
    }
  }
  
  public setPreviewStructure({ component }) {
    component.setValue(this.previewStructure);
  }
  
  get unitView() {
    return this.previewStructure === 1;
  }
  
  get detailedView() {
    return this.previewStructure === 2;
  }
  
  public cellIsEditableItem({ row }) {
    return row.NRSEQ !== 0;
  }
  
  public budgetUnits({ component }: any) {
    const routeName = 'budgetUnits';
    const sendObject = {};
    
    LoadingService.show();
    Http.post(`/${routeName}`, sendObject).then((response) => {
      component.datasource.setFilter([]);
      component.datasource.updateData(response.data.dataset[routeName]);
        
      LoadingService.hide();
    }).catch((e) => {
      // utilities.errorMessage(I18n.translate('gop::FAILED_DATA'));
      LoadingService.hide();
      console.warn(e);
    });
  }
  
  public routeDescriptionDetail() {
    return this.routeDescription;
  }
}
